// Author;
// SteveQuinn
//
// Status;
// Fully working and tested 07/07/16 14:37 
//
// Librarties Used;
// Requires MQTT PubSubClient Library found here:          https://github.com/knolleary/pubsubclient
// Requires Bounce2 Library found here:                    https://github.com/thomasfredericks/Bounce2
//
// Purpose;
// Controls study lights and uses MCP9700A to read enclosure internal temperature and Arduino Uno R3 with Ethernet Shield R3
// 
// Details;
// Inclusive of the above software support for MQTT over Ethernet it also implements the following functionality;
// 1. Reading software version
// 2. Reading local enclosure Temperature
// 3. Over temperature proctection and warning indication MQTT message
// 4. Control/Query of Swan, Shelf and Under Desk Lights in study
//
// Once switched on a resettable timer runs for approx 2hrs 30mins and then shuts all lights off. If another light is turned on the before
// timeout the period is extended by a further 2hrs 30mins. The timer duration is controlled by polling millis() in a non-blocking way
// Although the hardware has an I2C RTC it wasn't possible (not enough memory) to use the RTC with the Uno once the PubSubClient library 
// was integrated into the software, this is why millis() is used for time keeping. For the same reasons it was not possible to use DCHP or 
// mDNS either.
// If an MQTT broker cannot be found at the IP address stated in the script the software will still allow button press control and keep 
// polling for the presence of an MQTT broker.
// Enclosure temperature is polled every 10 minutes, if the temperature is > 1 degree different from the last read it will publish an MQTT
// update.
//
// Compliation details;
//
// Arduino IDE 1.6.9
//
// Using library SPI at version 1.0 in folder: C:\Program Files (x86)\Arduino\hardware\arduino\avr\libraries\SPI 
// Using library MQTT_Ethernet at version 2.6 in folder: C:\Users\User\Documents\Arduino\libraries\MQTT_Ethernet 
// Using library Ethernet at version 1.1.2 in folder: C:\Program Files (x86)\Arduino\libraries\Ethernet 
// Using library Bounce2 at version 2.1 in folder: C:\Users\User\Documents\Arduino\libraries\Bounce2 
// 
// Sketch uses 20,076 bytes (62%) of program storage space. Maximum is 32,256 bytes.
// Global variables use 1,456 bytes (71%) of dynamic memory, leaving 592 bytes for local variables. Maximum is 2,048 bytes.
//
//

//#define DEBUG_PRINT // Undefine this for comprehensive debug information via the serial port

#include <SPI.h>
#include <PubSubClient.h>
#include <Ethernet.h>
#include <Bounce2.h>
#include <limits.h>

// Struct to hold a single timer instance
typedef struct tsTimerInstance {
  void (*callback)(void);      // Function called when timing period exceeded
  boolean bRunning;            // Flag, set with timer running
  unsigned long ulTimerPeriod; // Timing period in milliseconds
  unsigned long ulStartValue;  // Grab of value from millis() when timer was started, used to calculate elapsed time
} TimerInstance;

#define TEMP_READ_HOURS        0
#define TEMP_READ_MINUTES      10
#define TEMP_READ_SECONDS      0
#define DELAY_HOURS            2
#define DELAY_MINUTES          30
#define DELAY_SECONDS          0
#define TOTAL_TIME_IN_MILLISECONDS(H, M, S) ((unsigned long)((((unsigned long)H)*60UL*60UL*1000UL)+(((unsigned long)M)*60UL*1000UL)+(((unsigned long)S)*1000UL)))
#define MAX_TIMERS             2
#define LIGHT_DELAY_TIMER      0
#define TEMP_READ_TIMER        1

TimerInstance stiTimerArray[MAX_TIMERS];  // Array for holding all the active timer instances

//#define MQTT_VERSION MQTT_VERSION_3_1
#define __FILENAME__ (strrchr(__FILE__, '\\') ? strrchr(__FILE__, '\\') + 1 : __FILE__)

const char hex[] = {"0123456789ABCDEF"};             // Used to convert a hex number to a Ascii representation of that value ie 0x0 => '0',... 0x0F => 'F'
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED }; // This IoT device's physical mac address
byte ip[]  = { W,  X,  Y,    Z };                    // This IoT device's IP address
IPAddress broker(W, X, Y, Z);                        // This is the IP address of the MQTT broker


EthernetClient ethClient;
PubSubClient client(ethClient);

char clientName[40]; // Used to hold the name of this client to register with the Broker, made up of GENERIC DEVICE NAME + '-' + MAC ADDRESS (in form XX:XX:XX:XX:XX:XX)
const char* THIS_GENERIC_DEVICE = "WIZNet5100";

// This device subscribes to this topic to receive commands to request software version (Generic Device, MAC Addr, Filename.ino)
// Replies are published on this topic 'swVerConfirmTopic (swVersion)'
const char* swVerTopic = "/EthernetDevice/SwVerCommand";

// This device publishes it's software version 'swVersion' to this topic. 
const char* swVerConfirmTopic = "/EthernetDevice/SwVerConfirm";

// This device subscribes to this topic to receive commands to request software version (Generic Device, MAC Addr, Filename.ino)
// Replies are published on this topic 'swVerConfirmTopic (swVersion)'. Command must be in form '/EthernetDevice/DE:AD:BE:EF:FE:ED/SwVerCommand'
char swVerThisDeviceTopic[70];

// Place holder for published replied on this topic 'swVerConfirmTopic' and is in the form (Generic Device, MAC Addr, Filename.ino)
// Replies are published on this topic 'swVerConfirmTopic'
char swVersion[80];

// This device subscribes to this topic to receive commands to query the Study light controller temperature status
const char* tempQueryTopic = "/EthernetDevice/Study/LightControllerTempQuery";

// This device publishes the Study Light Controller Internal Temperature to this topic  
const char* temperatureTopic = "/EthernetDevice/Study/LightControllerTemp";

// This device subscribes to this topic to receive commands to set Swan Lights on=1/off=0
const char* swanLightTopic = "/EthernetDevice/Study/SwanLight";

// This device subscribes to this topic to receive commands to set Top Shelf Lights on=1/off=0
const char* topShelfLightTopic = "/EthernetDevice/Study/TopShelfLight";

// This device subscribes to this topic to receive commands to set Under Desk Lights on=1/off=0 
const char* underDeskLightTopic = "/EthernetDevice/Study/UnderDeskLight";

// This device publishes to this topic to after receipt of a commands to set Swan,Top Shelf,Under Desk Lights on/off
const char* lightConfirmTopic = "/EthernetDevice/Study/LightConfirm";

// This device subscribes to this topic. 
// When recieved it publishes to lightConfirmTopic status of the Swan, Top Shelf and Under Desk Lights 
// Value ranges from 0...7. Bit 0 = Swan Light, Bit 1 = Top Shelf Light, Bit 2 = Under Desk Light. 1 = On, 0 = Off
const char* lightsQueryTopic = "/EthernetDevice/Study/LightsQuery";

// This device subscribes to this topic to receive commands to query the Study light controller over temp alarm status
const char* tempAlarmQueryTopic = "/EthernetDevice/Study/LightControllerTempAlarmQuery";

// This device publishes the Study Light controller temp alarm status to this topic after receipt. Active=1/Inactive=0
const char* tempAlarmConfirmTopic = "/EthernetDevice/Study/LightControllerTempAlarmConfirm";


/*
clientName = WIZNet5100-DE:AD:BE:EF:FE:ED
swVersion = WIZNet5100,DE:AD:BE:EF:FE:ED,MQTTStudyLightController1_8.ino
swVerThisDeviceTopic = /EthernetDevice/DE:AD:BE:EF:FE:ED/SwVerCommand

swVerTopic => swVerConfirmTopic (swVersion)
swVerThisDeviceTopic => swVerConfirmTopic (swVersion)

At Startup
client.publish(swVerConfirmTopic, swVersion);
*/



///////////////////
// Define inputs //
///////////////////
#define COMMON_BUTTON_DELAY_TIMES    20  // In milliseconds

// digital pin 8 
#define SWAN_LIGHT_BUTTON_DELAY_MS   COMMON_BUTTON_DELAY_TIMES   // Delay time for a button down to be recognised
#define SWAN_LIGHT_BIT_POSITION      0         // Use to shift bit position when responding to Query command to allow bit compression
#define SwanLightButtonPin           8         // I/P pin number
boolean       SwanLightButtonState = false;    // Records the current state of the light. Did this as not sure an o/p on Arduino can be read.
Bounce        SwanLightButton      = Bounce(); // Instance of debounced input button

// digital pin 7
#define TOP_SHELF_BUTTON_DELAY_MS    COMMON_BUTTON_DELAY_TIMES   // Delay time for a button down to be recognised
#define TOP_SHELF_BIT_POSITION       1         // Use to shift bit position when responding to Query command to allow bit compression
#define TopShelfButtonPin            7         // I/P pin number
boolean       TopShelfButtonState  = false;    // Records the current state of the light. Did this as not sure an o/p on Arduino can be read.
Bounce        TopShelfButton       = Bounce(); // Instance of debounced input button

// digital pin 2 
#define UNDER_DESK_BUTTON_DELAY_MS   COMMON_BUTTON_DELAY_TIMES   // Delay time for a button down to be recognised
#define UNDER_DESK_BIT_POSITION      2         // Use to shift bit position when responding to Query command to allow bit compression
#define UnderDeskButtonPin           2         // I/P pin number
boolean       UnderDeskButtonState = false;    // Records the current state of the light. Did this as not sure an o/p on Arduino can be read.
Bounce        UnderDeskButton      = Bounce(); // Instance of debounced input button

////////////////////
// Define outputs //
////////////////////
// digital pin 3
#define SwanLightRelayPin 3   // Output pin to control Swan Light Relay
// digital pin 5
#define TopShelfRelayPin  5   // Output pin to control Top Shelf Light Relay
// digital pin 6
#define UnderDeskRelayPin 6   // Output pin to control Under Desk Light Relay

//////////////////////
// Define Temp Vals //
//////////////////////
#define          OVER_TEMP      44.0   // Maximum internal over temperature value of light controller
#define          sensorPin      A0     // select the input pin for the thermistor
int            sensorValue    = 0;     // variable to store the raw volts coming off the thermistor
float      TemperatureDegC    = 0.0;
float   TemperatureDegCOld    = 0.0;
int     intTemperatureDegC    = 0;
int  intTemperatureDegCOld    = 0;
float fCalibrationConstant    = -1.98; // Constant to correct for temperature elevated value
//float fCalibrationConstant  = 0.0;
boolean    OverTempReached    = false; // Flag, set when light controller internal temp over safety limit
#define          MAX_TEMPS      30     // The number of temp readings to take an average over
float   averageTempArray[MAX_TEMPS];   // Array to store temp readings
byte    tempAverageArrayIndex = 0;     // Index pointer used to calculate rolling average of temperature

long  lastReconnectAttempt    = 0;     // To enable non-blocking MQTT reconnection
char tmpBuf[30];



void setup() {
  #ifdef DEBUG_PRINT
  Serial.begin(115200);
  while (!Serial) ; // wait for serial
  //Serial.println(__FILENAME__);  
  #endif
  TemperatureDegCOld = TemperatureDegC;

  // Configure I/O
  pinMode(SwanLightButtonPin, INPUT);
  SwanLightButtonState = false;
  SwanLightButton.attach(SwanLightButtonPin);
  SwanLightButton.interval(SWAN_LIGHT_BUTTON_DELAY_MS);
  SwanLightButton.update();
  
  pinMode(TopShelfButtonPin, INPUT);
  TopShelfButtonState = false;
  TopShelfButton.attach(TopShelfButtonPin);
  TopShelfButton.interval(TOP_SHELF_BUTTON_DELAY_MS);
  TopShelfButton.update();
  
  pinMode(UnderDeskButtonPin, INPUT);
  UnderDeskButtonState = false;
  UnderDeskButton.attach(UnderDeskButtonPin);
  UnderDeskButton.interval(UNDER_DESK_BUTTON_DELAY_MS);
  UnderDeskButton.update();
  
  pinMode(SwanLightRelayPin, OUTPUT);
  digitalWrite(SwanLightRelayPin,LOW);
  pinMode(TopShelfRelayPin, OUTPUT);
  digitalWrite(TopShelfRelayPin,LOW);
  pinMode(UnderDeskRelayPin, OUTPUT);
  digitalWrite(UnderDeskRelayPin,LOW);

  sprintf(clientName,"%s-%s",THIS_GENERIC_DEVICE,macToStr(mac,tmpBuf));
  sprintf(swVerThisDeviceTopic,"/EthernetDevice/%s/SwVerCommand",tmpBuf);
  sprintf(swVersion,"%s,%s,%s",THIS_GENERIC_DEVICE,tmpBuf,__FILENAME__);

  #ifdef DEBUG_PRINT
  Serial.print("clientName : ");
  Serial.println(clientName);
  Serial.print("swVersion : ");
  Serial.println(swVersion);
  Serial.print("swVerThisDeviceTopic : ");
  Serial.println(swVerThisDeviceTopic);
  #endif
  
  // MQTT Broker config
  client.setServer(broker, 1883);
  client.setCallback(callback);

  // Start Ethernet
  Ethernet.begin(mac, ip);  
  delay(1500);
  #ifdef DEBUG_PRINT
  Serial.println(Ethernet.localIP());
  #endif

  // set up average temp array
  TemperatureDegC = readTemp();
  TemperatureDegCOld = TemperatureDegC;
  intTemperatureDegC = TemperatureDegC;
  intTemperatureDegCOld = 0;
  tempAverageArrayIndex = 0;
  for (char x = 0; x < MAX_TEMPS; x++)
      averageTempArray[x] = TemperatureDegC;
      
  OverTempReached = false;
  lastReconnectAttempt = 0;

  timer_create(LIGHT_DELAY_TIMER, TOTAL_TIME_IN_MILLISECONDS(DELAY_HOURS,     DELAY_MINUTES,     DELAY_SECONDS),     TimerExpiryCallbackFunction);  
  timer_create(TEMP_READ_TIMER,   TOTAL_TIME_IN_MILLISECONDS(TEMP_READ_HOURS, TEMP_READ_MINUTES, TEMP_READ_SECONDS), UpdateTemperatureCallbackFunction);  
  timer_start(TEMP_READ_TIMER);
//while(1);  
}




void callback(char* topic, byte* payload, unsigned int length) {
  byte tmpByte=0;

  #ifdef DEBUG_PRINT
  Serial.print(F("Message arrived ["));
  Serial.print(topic);
  Serial.print(F("] "));
  for (int i=0;i<length;i++) {
    Serial.print((char)payload[i]);
  }
  Serial.println();
  #endif


  if (strcmp(lightsQueryTopic,topic)== 0) {
    tmpByte = ( ((UnderDeskButtonState?1:0)<<UNDER_DESK_BIT_POSITION) | ((TopShelfButtonState?1:0)<<TOP_SHELF_BIT_POSITION) | ((SwanLightButtonState?1:0)<<SWAN_LIGHT_BIT_POSITION) );
    #ifdef DEBUG_PRINT
    Serial.print(F("Light Status :"));
    Serial.print(tmpByte);
    #endif
    tmpBuf[0] = (char)(tmpByte + '0');
    tmpBuf[1] = 0;
    client.publish(lightConfirmTopic, tmpBuf);
    return;
  }  

  if (strcmp(swanLightTopic,topic)== 0) {
    if(payload[0] == '1'){ //turn the light on if the payload is '1' and publish the confirmation 
      digitalWrite(SwanLightRelayPin, HIGH);
      //client.publish(lightConfirmTopic, "SOn");
      SwanLightButtonState = true;
      if (timer_isRunning(LIGHT_DELAY_TIMER))
        timer_reset(LIGHT_DELAY_TIMER);
      else  
        timer_start(LIGHT_DELAY_TIMER);
    } else if (payload[0] == '0'){ //turn the light off if the payload is '0' and publish the confirmation
      digitalWrite(SwanLightRelayPin, LOW);
      //client.publish(lightConfirmTopic, "SOff");
      SwanLightButtonState = false;
      if ((timer_isRunning(LIGHT_DELAY_TIMER)) && (!TopShelfButtonState) && (!UnderDeskButtonState))
        timer_stop(LIGHT_DELAY_TIMER);
    } else {
      client.publish(lightConfirmTopic, "SErr");
    }
    tmpByte = ( ((UnderDeskButtonState?1:0)<<UNDER_DESK_BIT_POSITION) | ((TopShelfButtonState?1:0)<<TOP_SHELF_BIT_POSITION) | ((SwanLightButtonState?1:0)<<SWAN_LIGHT_BIT_POSITION) );
    tmpBuf[0] = (char)(tmpByte + '0');
    tmpBuf[1] = 0;
    client.publish(lightConfirmTopic, tmpBuf);
    return;
  }

  if (strcmp(topShelfLightTopic,topic)== 0) {
    if(payload[0] == '1'){ //turn the light on if the payload is '1' and publish the confirmation 
      digitalWrite(TopShelfRelayPin, HIGH);
      //client.publish(lightConfirmTopic, "TOn");
      TopShelfButtonState = true;
      if (timer_isRunning(LIGHT_DELAY_TIMER))
        timer_reset(LIGHT_DELAY_TIMER);
      else  
        timer_start(LIGHT_DELAY_TIMER);
    } else if (payload[0] == '0'){ //turn the light off if the payload is '0' and publish the confirmation
      digitalWrite(TopShelfRelayPin, LOW);
      //client.publish(lightConfirmTopic, "TOff");
      TopShelfButtonState = false;
      if ((timer_isRunning(LIGHT_DELAY_TIMER)) && (!SwanLightButtonState) && (!UnderDeskButtonState))
        timer_stop(LIGHT_DELAY_TIMER);
    } else {
      client.publish(lightConfirmTopic, "TErr");
    }
    tmpByte = ( ((UnderDeskButtonState?1:0)<<UNDER_DESK_BIT_POSITION) | ((TopShelfButtonState?1:0)<<TOP_SHELF_BIT_POSITION) | ((SwanLightButtonState?1:0)<<SWAN_LIGHT_BIT_POSITION) );
    tmpBuf[0] = (char)(tmpByte + '0');
    tmpBuf[1] = 0;
    client.publish(lightConfirmTopic, tmpBuf);
    return;
  }

  if (strcmp(underDeskLightTopic,topic)== 0) {
    if(payload[0] == '1'){ //turn the light on if the payload is '1' and publish the confirmation 
      digitalWrite(UnderDeskRelayPin, HIGH);
      //client.publish(lightConfirmTopic, "UOn");
      UnderDeskButtonState = true;
      if (timer_isRunning(LIGHT_DELAY_TIMER))
        timer_reset(LIGHT_DELAY_TIMER);
      else  
        timer_start(LIGHT_DELAY_TIMER);
    } else if (payload[0] == '0'){ //turn the light off if the payload is '0' and publish the confirmation
      digitalWrite(UnderDeskRelayPin, LOW);
      //client.publish(lightConfirmTopic, "UOff");
      UnderDeskButtonState = false;
      if ((timer_isRunning(LIGHT_DELAY_TIMER)) && (!SwanLightButtonState) && (!TopShelfButtonState))
        timer_stop(LIGHT_DELAY_TIMER);
    } else {
      client.publish(lightConfirmTopic, "UErr");
    }
    tmpByte = ( ((UnderDeskButtonState?1:0)<<UNDER_DESK_BIT_POSITION) | ((TopShelfButtonState?1:0)<<TOP_SHELF_BIT_POSITION) | ((SwanLightButtonState?1:0)<<SWAN_LIGHT_BIT_POSITION) );
    tmpBuf[0] = (char)(tmpByte + '0');
    tmpBuf[1] = 0;
    client.publish(lightConfirmTopic, tmpBuf);
    return;
  }

  if (strcmp(swVerTopic,topic)== 0) {
    client.publish(swVerConfirmTopic, swVersion);
    return;
  }  

  if (strcmp(swVerThisDeviceTopic,topic)== 0) {
    client.publish(swVerConfirmTopic, swVersion);
    return;
  }  

  if (strcmp(tempAlarmQueryTopic,topic)== 0) {
    #ifdef DEBUG_PRINT
    Serial.print(F("Over Temp Status :"));
    Serial.print(OverTempReached);
    #endif
    tmpBuf[0] = (char)(OverTempReached?'1':'0');
    tmpBuf[1] = 0;
    client.publish(tempAlarmConfirmTopic, tmpBuf);
    return;
  }  

  if (strcmp(tempQueryTopic,topic)== 0) {
    sprintf(tmpBuf,"%ld.%02ld",(unsigned long)(TemperatureDegC), ((unsigned long)(((float)(TemperatureDegC - ((float)(intTemperatureDegC))))*100)));
    client.publish(temperatureTopic, tmpBuf);
    #ifdef DEBUG_PRINT
    Serial.print(F("Temp Status :"));
    Serial.print(tmpBuf);
    #endif
    return;
  }  
}




void loop() {
  checkButtons();
  timer_update();
  if (!client.connected()) {
    long now = millis();
    if (now - lastReconnectAttempt > 5000) {
      lastReconnectAttempt = now;
      // Attempt to reconnect
      if (reconnect()) {
        lastReconnectAttempt = 0;
      }
    }
  } else {
    // Client connected
    client.loop();
    checkTemperature();
  }
}


boolean reconnect(void) {
  #ifdef DEBUG_PRINT
  Serial.print(F("Attempting MQTT connection..."));
  #endif
  // Attempt to connect
  if (client.connect(clientName)) {
    #ifdef DEBUG_PRINT
    Serial.println(F("connected"));
    #endif
    // Software version subscriptions
    client.publish(swVerConfirmTopic, swVersion);        
    client.subscribe(swVerThisDeviceTopic);
    client.subscribe(swVerTopic);
    // Light state subscription
    client.subscribe(lightsQueryTopic);
    client.subscribe(swanLightTopic);
    client.subscribe(topShelfLightTopic);
    client.subscribe(underDeskLightTopic);
    // Study Light Controller Temperature Status Query topic subscription
    client.subscribe(tempQueryTopic);
    client.subscribe(tempAlarmQueryTopic);
  }
}



void checkButtons(void)
{
  byte tmpByte=0;
  #ifdef DEBUG_PRINT
  //Serial.println(F("checkButtons"));
  #endif
  if ((SwanLightButton.update()) && (SwanLightButton.read() == LOW)) // Value reads low if button pressed
  {
     if(SwanLightButtonState == true){
       digitalWrite(SwanLightRelayPin,LOW);
       //client.publish(lightConfirmTopic, "SOff");
       SwanLightButtonState = false;
       if ((timer_isRunning(LIGHT_DELAY_TIMER)) && (!TopShelfButtonState) && (!UnderDeskButtonState))
         timer_stop(LIGHT_DELAY_TIMER);
     } else {
       digitalWrite(SwanLightRelayPin,HIGH);
       //client.publish(lightConfirmTopic, "SOn");
       SwanLightButtonState = true;
       if (timer_isRunning(LIGHT_DELAY_TIMER))
         timer_reset(LIGHT_DELAY_TIMER);
       else  
         timer_start(LIGHT_DELAY_TIMER);
     }
     tmpByte = ( ((UnderDeskButtonState?1:0)<<UNDER_DESK_BIT_POSITION) | ((TopShelfButtonState?1:0)<<TOP_SHELF_BIT_POSITION) | ((SwanLightButtonState?1:0)<<SWAN_LIGHT_BIT_POSITION) );
     tmpBuf[0] = (char)(tmpByte + '0');
     tmpBuf[1] = 0;
     client.publish(lightConfirmTopic, tmpBuf);
  }
  
  if(TopShelfButton.update() && TopShelfButton.read() == LOW){ // Value reads low if button pressed
    if(TopShelfButtonState == true){
      digitalWrite(TopShelfRelayPin,LOW);
      //client.publish(lightConfirmTopic, "TOff");
      TopShelfButtonState = false;
      if ((timer_isRunning(LIGHT_DELAY_TIMER)) && (!SwanLightButtonState) && (!UnderDeskButtonState))
        timer_stop(LIGHT_DELAY_TIMER);
    } else {
        digitalWrite(TopShelfRelayPin,HIGH);
      //client.publish(lightConfirmTopic, "TOn");
      TopShelfButtonState = true;
       if (timer_isRunning(LIGHT_DELAY_TIMER))
         timer_reset(LIGHT_DELAY_TIMER);
       else  
         timer_start(LIGHT_DELAY_TIMER);
    }
    tmpByte = ( ((UnderDeskButtonState?1:0)<<UNDER_DESK_BIT_POSITION) | ((TopShelfButtonState?1:0)<<TOP_SHELF_BIT_POSITION) | ((SwanLightButtonState?1:0)<<SWAN_LIGHT_BIT_POSITION) );
    tmpBuf[0] = (char)(tmpByte + '0');
    tmpBuf[1] = 0;
    client.publish(lightConfirmTopic, tmpBuf);
  }

  if(UnderDeskButton.update() && UnderDeskButton.read() == LOW){ // Value reads low if button pressed
    if(UnderDeskButtonState == true){
      digitalWrite(UnderDeskRelayPin,LOW);
      //client.publish(lightConfirmTopic, "UOff");
      UnderDeskButtonState = false;
      if ((timer_isRunning(LIGHT_DELAY_TIMER)) && (!SwanLightButtonState) && (!TopShelfButtonState))
        timer_stop(LIGHT_DELAY_TIMER);
    } else {
      digitalWrite(UnderDeskRelayPin,HIGH);
      //client.publish(lightConfirmTopic, "UOn");
      UnderDeskButtonState = true;
      if (timer_isRunning(LIGHT_DELAY_TIMER))
        timer_reset(LIGHT_DELAY_TIMER);
      else  
        timer_start(LIGHT_DELAY_TIMER);
    }
    tmpByte = ( ((UnderDeskButtonState?1:0)<<UNDER_DESK_BIT_POSITION) | ((TopShelfButtonState?1:0)<<TOP_SHELF_BIT_POSITION) | ((SwanLightButtonState?1:0)<<SWAN_LIGHT_BIT_POSITION) );
    tmpBuf[0] = (char)(tmpByte + '0');
    tmpBuf[1] = 0;
    client.publish(lightConfirmTopic, tmpBuf);
  }
}


float readTemp(void)
{
  sensorValue = analogRead(sensorPin);
  return ((((((float)sensorValue) * (5.0 / 1024.0))-0.5)/0.01)+ fCalibrationConstant);
}


void UpdateTemperatureCallbackFunction(void)
{
  if (intTemperatureDegCOld != intTemperatureDegC)
  {
    sprintf(tmpBuf,"%ld.%02ld",(unsigned long)(TemperatureDegC), ((unsigned long)(((float)(TemperatureDegC - ((float)(intTemperatureDegC))))*100)));
    client.publish(temperatureTopic, tmpBuf);
    intTemperatureDegCOld = intTemperatureDegC;
    TemperatureDegCOld = TemperatureDegC;

    #ifdef DEBUG_PRINT
    Serial.print(F("TemperatureDegC : ")); //print to serial monitor for debuging
    Serial.println(TemperatureDegC);
    Serial.print(F("intTemperatureDegC : ")); //print to serial monitor for debuging
    Serial.println(intTemperatureDegC);
    Serial.print(F("intTemperatureDegCOld : ")); //print to serial monitor for debuging
    Serial.println(intTemperatureDegCOld);
    Serial.print(F("tmpBuf : ")); //print to serial monitor for debuging
    Serial.println(tmpBuf);
    #endif
  }
  timer_start(TEMP_READ_TIMER);
}


void checkTemperature(void)
{
//  sensorValue = analogRead(sensorPin);
//  TemperatureDegC = (((((float)sensorValue) * (5.0 / 1024.0))-0.5)/0.01);

  TemperatureDegC = readTemp();

  averageTempArray[tempAverageArrayIndex++] = readTemp();
  if (tempAverageArrayIndex >= MAX_TEMPS) tempAverageArrayIndex = 0;
  
  for (char x = 0; x < MAX_TEMPS; x++)
      TemperatureDegC += averageTempArray[x];

  TemperatureDegC = (float)(TemperatureDegC / ((float)MAX_TEMPS));
  intTemperatureDegC = TemperatureDegC;
  

  if (TemperatureDegC >= OVER_TEMP)
    if (!OverTempReached) {
      OverTempReached = true;
      client.publish(tempAlarmConfirmTopic, "1\0");
    }  
  else {
    OverTempReached = false;
  }
  
  if (OverTempReached)  // Temp in box too high. Turn off all relays as they consume an extra 30mA each
    TurnOffAllRelays();
}


// Convert MAC addr to a string
char * macToStr(const uint8_t* mac, char * dest){
  char *src = dest;
  for (int i = 0; i < 6; ++i) {
    *dest++ = hex[((mac[i] & 0xF0)>>4)]; 
    *dest++ = hex[(mac[i] & 0x0F)]; 
    if (i < 5) *dest++ = ':';
  }
  *dest = 0x00;
  dest = src;
  return dest;
}


void TurnOffAllRelays(void)
{
  byte tmpByte = 0;
  digitalWrite(SwanLightRelayPin,LOW);
  SwanLightButtonState = false;
  //client.publish(lightConfirmTopic, "SOff");

  digitalWrite(TopShelfRelayPin,LOW);
  TopShelfButtonState  = false;
  //client.publish(lightConfirmTopic, "TOff");

  digitalWrite(UnderDeskRelayPin,LOW);
  UnderDeskButtonState = false;
  //client.publish(lightConfirmTopic, "UOff");
  
  tmpByte = ( ((UnderDeskButtonState?1:0)<<UNDER_DESK_BIT_POSITION) | ((TopShelfButtonState?1:0)<<TOP_SHELF_BIT_POSITION) | ((SwanLightButtonState?1:0)<<SWAN_LIGHT_BIT_POSITION) );
  tmpBuf[0] = (char)(tmpByte + '0');
  tmpBuf[1] = 0;
  client.publish(lightConfirmTopic, tmpBuf);
  
}


void TimerExpiryCallbackFunction(void)
{
  byte tmpByte = 0;
  #ifdef DEBUG_PRINT
  Serial.print(F("T Ex : "));
  #endif
  if (SwanLightButtonState) {
    #ifdef DEBUG_PRINT
    Serial.print(F("Swan "));
    #endif
    SwanLightButtonState = false;
    digitalWrite(SwanLightRelayPin,LOW);
    //client.publish(lightConfirmTopic, "SOff");
  }
  
  if (TopShelfButtonState) {
    #ifdef DEBUG_PRINT
    Serial.print(F("Top "));
    #endif
    TopShelfButtonState  = false;
    digitalWrite(TopShelfRelayPin,LOW);
    //client.publish(lightConfirmTopic, "TOff");
  }  
  
  if (UnderDeskButtonState) {
    #ifdef DEBUG_PRINT
    Serial.print(F("Under "));
    #endif
    UnderDeskButtonState = false;
    digitalWrite(UnderDeskRelayPin,LOW);
    //client.publish(lightConfirmTopic, "UOff");
  }
  tmpByte = ( ((UnderDeskButtonState?1:0)<<UNDER_DESK_BIT_POSITION) | ((TopShelfButtonState?1:0)<<TOP_SHELF_BIT_POSITION) | ((SwanLightButtonState?1:0)<<SWAN_LIGHT_BIT_POSITION) );
  tmpBuf[0] = (char)(tmpByte + '0');
  tmpBuf[1] = 0;
  client.publish(lightConfirmTopic, tmpBuf);
  #ifdef DEBUG_PRINT
  Serial.println();
  #endif
}


void timer_create(int iTimerNumber, unsigned long ulTimerPeriod, void (*callbackfn)(void))
{
  if (iTimerNumber <= MAX_TIMERS)
  {
    stiTimerArray[iTimerNumber].callback = callbackfn;
    stiTimerArray[iTimerNumber].bRunning = false;
    stiTimerArray[iTimerNumber].ulTimerPeriod = ulTimerPeriod;
    stiTimerArray[iTimerNumber].ulStartValue = 0;
    #ifdef DEBUG_PRINT
    Serial.print(F("T Create, TNum : "));
    Serial.print(iTimerNumber);
    Serial.print(F(", TPeriod : "));
    Serial.println(ulTimerPeriod);
    #endif
  }
}


void timer_update(void)
{
  unsigned long ulCurrentTime = millis();
  unsigned long ulElapsedTime = 0;
  
  for (int iIndex = 0; iIndex < MAX_TIMERS; iIndex++)
  {
    if (stiTimerArray[iIndex].bRunning)
    {
      ulElapsedTime = ulCurrentTime - stiTimerArray[iIndex].ulStartValue;
      /* // Argh! twos complement arithmetic, I hate it...
      if (ulCurrentTime < stiTimerArray[iIndex].ulStartValue) // Cater for UL counter wrap ~every day
        ulElapsedTime = ulCurrentTime - stiTimerArray[iIndex].ulStartValue;
      else  
        ulElapsedTime = ulCurrentTime + (ULONG_MAX - stiTimerArray[iIndex].ulStartValue);
      */
      #ifdef DEBUG_PRINT
      Serial.print(F("T Up, TNum : "));
      Serial.print(iIndex);
      Serial.print(F(", T Elapsed : "));
      Serial.println(ulElapsedTime);
      #endif
        
      if (ulElapsedTime >= stiTimerArray[iIndex].ulTimerPeriod)
      {
        stiTimerArray[iIndex].bRunning = false;
        stiTimerArray[iIndex].callback();
      }
    }
  }
}


void timer_start(int iTimerNumber)
{
  if (iTimerNumber <= MAX_TIMERS)
  {
    stiTimerArray[iTimerNumber].ulStartValue = millis();
    stiTimerArray[iTimerNumber].bRunning = true;
    #ifdef DEBUG_PRINT
    Serial.print(F("T Start , TNum : "));
    Serial.print(iTimerNumber);
    Serial.print(F(", TStart : "));
    Serial.println(stiTimerArray[iTimerNumber].ulStartValue);
    #endif
  }
}


void timer_stop(int iTimerNumber)
{
  if (iTimerNumber <= MAX_TIMERS)
    stiTimerArray[iTimerNumber].bRunning = false;
  #ifdef DEBUG_PRINT
  Serial.print(F("T Stop : "));
  Serial.println(iTimerNumber);
  #endif
}


void timer_reset(int iTimerNumber)
{
  if (iTimerNumber <= MAX_TIMERS)
    stiTimerArray[iTimerNumber].ulStartValue = millis();
  #ifdef DEBUG_PRINT
  Serial.print(F("T Reset : "));
  Serial.println(iTimerNumber);
  #endif
}


boolean timer_isRunning(int iTimerNumber)
{
  return stiTimerArray[iTimerNumber].bRunning;
}

